@@ -6,15 +6,18 @@ tmuxp.workspacebuilder
 
 """
 
-from __future__ import absolute_import, division, print_function, \
-    with_statement, unicode_literals
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals, with_statement)
 
-import os
 import logging
-import subprocess
 
-from . import exc, config, Window, Pane, Session, Server
-from ._compat import PY2, console_to_str
+from libtmux.exc import TmuxSessionExists
+from libtmux.pane import Pane
+from libtmux.server import Server
+from libtmux.session import Session
+from libtmux.window import Window
+
+from . import exc
 from .util import run_before_script
 
 logger = logging.getLogger(__name__)
@@ -91,6 +94,18 @@ class WorkspaceBuilder(object):
 
         self.sconf = sconf
 
+    def session_exists(self, session_name=None):
+        exists = self.server.has_session(session_name)
+        if not exists:
+            return exists
+
+        self.session = self.server.find_where(
+            {
+                'session_name': session_name
+            }
+        )
+        return True
+
     def build(self, session=None):
         """Build tmux workspace in session.
 
@@ -112,12 +127,12 @@ class WorkspaceBuilder(object):
                 )
 
             if self.server.has_session(self.sconf['session_name']):
-                self.session = self.server.findWhere(
+                self.session = self.server.find_where(
                     {
                         'session_name': self.sconf['session_name']
                     }
                 )
-                raise exc.TmuxSessionExists(
+                raise TmuxSessionExists(
                     'Session name %s is already running.' %
                     self.sconf['session_name']
                 )
@@ -126,15 +141,15 @@ class WorkspaceBuilder(object):
                     session_name=self.sconf['session_name']
                 )
 
-            assert(self.sconf['session_name'] == session.get('session_name'))
+            assert(self.sconf['session_name'] == session.name)
             assert(len(self.sconf['session_name']) > 0)
 
         self.session = session
         self.server = session.server
 
         self.server._list_sessions()
-        assert self.server.has_session(session.get('session_name'))
-        assert session.get('session_id')
+        assert self.server.has_session(session.name)
+        assert session.id
 
         assert(isinstance(session, Session))
 
@@ -145,7 +160,16 @@ class WorkspaceBuilder(object):
                 run_before_script(self.sconf['before_script'])
             except Exception as e:
                 self.session.kill_session()
-                raise(e)
+                raise e
+        if 'options' in self.sconf:
+            for option, value in self.sconf['options'].items():
+                self.session.set_option(option, value)
+        if 'global_options' in self.sconf:
+            for option, value in self.sconf['global_options'].items():
+                self.session.set_option(option, value, g=True)
+        if 'environment' in self.sconf:
+            for option, value in self.sconf['environment'].items():
+                self.session.set_environment(option, value)
 
         for w, wconf in self.iter_create_windows(session):
             assert(isinstance(w, Window))
@@ -190,16 +214,26 @@ class WorkspaceBuilder(object):
 
             w1 = None
             if i == int(1):  # if first window, use window 1
-                w1 = s.attached_window()
+                w1 = s.attached_window
                 w1.move_window(99)
                 pass
 
-            sd = wconf['start_directory'] if 'start_directory' in wconf else None
+            if 'start_directory' in wconf:
+                sd = wconf['start_directory']
+            else:
+                sd = None
+
+            if 'window_shell' in wconf:
+                ws = wconf['window_shell']
+            else:
+                ws = None
+
             w = s.new_window(
                 window_name=window_name,
                 start_directory=sd,
                 attach=False,  # do not move to the new window
                 window_index=wconf.get('window_index', ''),
+                window_shell=ws,
             )
 
             if i == int(1) and w1:  # if first window, use window 1
@@ -236,7 +270,7 @@ class WorkspaceBuilder(object):
         for pindex, pconf in enumerate(wconf['panes'], start=pane_base_index):
 
             if pindex == int(pane_base_index):
-                p = w.attached_pane()
+                p = w.attached_pane
 
             else:
                 def get_pane_start_directory():
@@ -253,12 +287,19 @@ class WorkspaceBuilder(object):
                 )
 
             assert(isinstance(p, Pane))
-            assert(int(p.get('pane_index')) == int(pindex))
+            assert(int(p.index) == int(pindex))
             if 'layout' in wconf:
                 w.select_layout(wconf['layout'])
 
+            if 'suppress_history' in pconf:
+                suppress = pconf['suppress_history']
+            elif 'suppress_history' in wconf:
+                suppress = wconf['suppress_history']
+            else:
+                suppress = True
+
             for cmd in pconf['shell_command']:
-                p.send_keys(cmd)
+                p.send_keys(cmd, suppress_history=suppress)
 
             if 'focus' in pconf and pconf['focus']:
                 w.select_pane(p['pane_id'])
@@ -276,43 +317,41 @@ def freeze(session):
     :rtype: dict
 
     """
-    sconf = {}
+    sconf = {'session_name': session['session_name'], 'windows': []}
 
-    sconf['session_name'] = session['session_name']
-
-    sconf['windows'] = []
     for w in session.windows:
-        wconf = {}
-        wconf['options'] = w.show_window_options()
-        wconf['window_name'] = w.get('window_name')
-        wconf['layout'] = w.get('window_layout')
-        wconf['panes'] = []
+        wconf = {
+            'options': w.show_window_options(),
+            'window_name': w.name,
+            'layout': w.layout,
+            'panes': []
+        }
         if w.get('window_active', '0') == '1':
-            wconf['focus']='true'
+            wconf['focus'] = 'true'
 
         # If all panes have same path, set 'start_directory' instead
         # of using 'cd' shell commands.
-        pane_has_same_path = lambda p: (
-            w.panes[0].get('pane_current_path') ==
-            p.get('pane_current_path')
-        )
+        def pane_has_same_path(p):
+            return (
+                w.panes[0].current_path ==
+                p.current_path
+            )
 
-        if (all(pane_has_same_path(p) for p in w.panes)):
-            wconf['start_directory'] = w.panes[0].get('pane_current_path')
+        if all(pane_has_same_path(p) for p in w.panes):
+            wconf['start_directory'] = w.panes[0].current_path
 
         for p in w.panes:
-            pconf = {}
-            pconf['shell_command'] = []
+            pconf = {'shell_command': []}
 
             if 'start_directory' not in wconf:
                 pconf['shell_command'].append(
-                    'cd ' + p.get('pane_current_path')
+                    'cd ' + p.current_path
                 )
-                
-            if p.get('pane_active', '0') == '1':
-                pconf['focus']='true'
 
-            current_cmd = p.get('pane_current_command')
+            if p.get('pane_active', '0') == '1':
+                pconf['focus'] = 'true'
+
+            current_cmd = p.current_command
 
             def filter_interpretters_and_shells():
                 return (
@@ -323,7 +362,7 @@ def freeze(session):
                     )
                 )
 
-            if (filter_interpretters_and_shells()):
+            if filter_interpretters_and_shells():
                 current_cmd = None
 
             if current_cmd:
